'
'
' ####################
' #####  PROLOG  #####
' ####################
'
' Demo program for LogLog and LinearLinear plot domains.
'
PROGRAM	"domain"
VERSION	"0.0001"
CONSOLE

'	IMPORT	"xst"   ' Standard library : required by most programs
'	IMPORT	"xio"		' Console IO library
	IMPORT	"cpdf"	' cpdf.dll
	IMPORT	"msvcrt"

DECLARE FUNCTION  Entry ()
DECLARE FUNCTION  generateGaussianData ()
DECLARE FUNCTION  plot_Curve (pdf)
DECLARE FUNCTION  do_LinearLinear (pdf)
DECLARE FUNCTION  do_LogLog (pdf)
DECLARE FUNCTION  do_myTitle (pdf)
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION  Entry ()

' ***** Initialization *****

	pdf = cpdf_open (0, NULL)
	cpdf_enableCompression (pdf, $$YES)												' use Flate/Zlib compression
	cpdf_init (pdf)
	cpdf_pageInit (pdf, 1, $$PORTRAIT, &$$LETTER, &$$LETTER)	' page orientation

	cpdf_setCreator (pdf, &"ClibPDF Domain Demo: domain.x")

' ***** Your drawing code in PDF below *****
	generateGaussianData ()
	do_LinearLinear (pdf)
	do_LogLog (pdf)
	do_myTitle (pdf)

	cpdf_finalizeAll (pdf)			' PDF file is actually written here

' save generated PDF to file
	cpdf_savePDFmemoryStreamToFile (pdf, &"domain.pdf")

' ***** Clean up *****

	cpdf_launchPreview (pdf)	' launch Acrobat/PDF viewer on the output file
	cpdf_close (pdf)					' shut down

END FUNCTION
'
'
' #####################################
' #####  generateGaussianData ()  #####
' #####################################
'
FUNCTION  generateGaussianData ()

	SINGLE xp, xbump
	SINGLE A, X0, Sigma
	SHARED SINGLE x[], y[]

' ***** Fit parameter definition for Gaussian:  y = A*exp( - ((x - X0)/Sigma)^2) *****

	A = 70.5
	X0 = 1.2
	Sigma = 0.4

' ***** computed Gaussian data *****

	DIM x[120]
	DIM y[120]

	xp = 0.03
	xbump = 0.03

	FOR i = 0 TO 99
' ***** A Gaussian curve *****
		x[i] = xp
		y[i] = A * exp( - ((xp - X0) / Sigma) * ((xp - X0) / Sigma))
		xp = xp + xbump
	NEXT i

END FUNCTION
'
'
' ################################
' #####  do_LinearLinear ()  #####
' ################################
'
FUNCTION  do_LinearLinear (pdf)

	SINGLE ymax, ymaxL, ymin, xmax, xmin

	ymax = 100.0
	ymaxL = 80.0
	ymin = 0.01
	xmax = 3.0
	xmin = 0.03

	myDomain = cpdf_createPlotDomain (1.5*$$inch, 1.5*$$inch, 4.5*$$inch, 3.0*$$inch,	0.0, xmax, 0.0, ymaxL, $$LINEAR, $$LINEAR, 0)
	oldDomain = cpdf_setPlotDomain (pdf, myDomain)				' save oldDomain for later restore
	cpdf_fillDomainWithRGBcolor (myDomain, 1.0, 0.9, 0.9)	' light pink
	cpdf_drawMeshForDomain (myDomain)

	cpdf_setgray (pdf, 0.0)

' X-Axis ---------------------------------------------------------------------------------
	xAxis = cpdf_createAxis (0.0, 4.5*$$inch, $$LINEAR, 0.0, xmax)
	cpdf_attachAxisToDomain (xAxis, myDomain, 0.0, -0.2*$$inch)
	cpdf_setAxisNumberFormat (xAxis, &"%g", &"Helvetica", 16.0)
	cpdf_setAxisLabel (xAxis, &"X axis (linear)", &"Times-Roman", &"MacRomanEncoding", 20.0)
	cpdf_drawAxis (xAxis)
	cpdf_freeAxis (xAxis)

' Y-Axis ---------------------------------------------------------------------------------
	yAxis = cpdf_createAxis (90.0, 3.0*$$inch, $$LINEAR, 0.0, ymaxL)
	cpdf_attachAxisToDomain (yAxis, myDomain, -0.2*$$inch, 0.0)
	cpdf_setAxisNumberFormat (yAxis, &"%2.f", &"Helvetica", 16.0)
	cpdf_setAxisLabel (yAxis, &"Y axis (linear)", &"Times-Roman", &"MacRomanEncoding", 20.0)
	cpdf_drawAxis (yAxis)
	cpdf_freeAxis (yAxis)

' ***** Plot it *****
	plot_Curve (pdf)

	cpdf_setPlotDomain (pdf, oldDomain)		' restore previous plot domain
	cpdf_freePlotDomain (myDomain)				' deallocate the plot domain

END FUNCTION
'
'
' ##########################
' #####  do_LogLog ()  #####
' ##########################
'
FUNCTION  do_LogLog (pdf)

	SINGLE ymax, ymaxL, ymin, xmax, xmin

	ymax = 100.0
	ymaxL = 80.0
	ymin = 0.01
	xmax = 3.0
	xmin = 0.03

	myDomain = cpdf_createPlotDomain (1.5*$$inch, 6.0*$$inch, 4.5*$$inch, 3.0*$$inch,	xmin, xmax, ymin, ymax, $$LOGARITHMIC, $$LOGARITHMIC, 0)
	oldDomain = cpdf_setPlotDomain (pdf, myDomain)					' save oldDomain for later restore
	cpdf_fillDomainWithRGBcolor (myDomain, 1.0, 0.9, 0.9)		' light pink
	cpdf_drawMeshForDomain (myDomain)

	cpdf_setgray (pdf, 0.0)

' X-Axis ---------------------------------------------------------------------------------
	xAxis = cpdf_createAxis (0.0, 4.5*$$inch, $$LOGARITHMIC, xmin, xmax)
	cpdf_setLogAxisNumberSelector (xAxis, $$LOGAXSEL_13)				' Nums 1 and 3s
	cpdf_attachAxisToDomain (xAxis, myDomain, 0.0, -0.2*$$inch)
	cpdf_setAxisNumberFormat (xAxis, &"%g", &"Helvetica", 16.0)
	cpdf_setAxisLabel (xAxis, &"X axis (logarithmic)", &"Times-Roman", &"MacRomanEncoding", 20.0)
	cpdf_drawAxis (xAxis)
	cpdf_freeAxis (xAxis)

' Y-Axis ---------------------------------------------------------------------------------
	yAxis = cpdf_createAxis (90.0, 3.0*$$inch, $$LOGARITHMIC, ymin, ymax)
	cpdf_attachAxisToDomain (yAxis, myDomain, -0.2*$$inch, 0.0)
	cpdf_setAxisNumberFormat (yAxis, &"%g", &"Helvetica", 16.0)
	cpdf_setAxisLabel(yAxis, &"Y axis (logarithmic)", &"Times-Roman", &"MacRomanEncoding", 20.0)
	cpdf_drawAxis (yAxis)
	cpdf_freeAxis (yAxis)

' We have to do clipping of domain, because log plot can under-shoot
	cpdf_gsave (pdf)							' to later undo clipping
	cpdf_clipDomain (myDomain)		' clip to myDomain: TRY it without this line
	plot_Curve (pdf)							' Plot it
	cpdf_grestore (pdf)						' remove clipping

	cpdf_setPlotDomain (pdf, oldDomain)		' restore previous plot domain
	cpdf_freePlotDomain (myDomain)				' deallocate the plot domain

END FUNCTION
'
'
' ###########################
' #####  do_myTitle ()  #####
' ###########################
'
FUNCTION  do_myTitle (pdf)

	cpdf_setrgbcolorFill (pdf, 0.0, 0.0, 1.0)		' Blue
	cpdf_beginText (pdf, 0)
	cpdf_setFont (pdf, &"Times-Italic", &"MacRomanEncoding", 18.0)
	cpdf_text (pdf, 1.5, 9.7, 0.0, &"Y = A * exp( - ((X-Xo)/S)^2)")
	cpdf_text (pdf, 1.7, 9.4, 0.0, &"where  A=70.5,  Xo=1.2,  S=0.4")
	cpdf_endText (pdf)

END FUNCTION
'
'
' ###########################
' #####  plot_Curve ()  #####
' ###########################
'
' This will be called from both do_LinearLinear(), and do_LogLog().
' Respective domains will take care of transformation and scaling.
' So, we just use the raw data values.
'
FUNCTION  plot_Curve (pdf)

	SHARED SINGLE x[], y[]

	cpdf_comments (pdf, &"\n%% plot the curve (linear-linear).\n")
	cpdf_setlinewidth (pdf, 2.0)
	cpdf_setrgbcolorStroke (pdf, 0.0, 0.0, 1.0)	' blue
	FOR i = 0 TO 99
		IFZ i THEN
			cpdf_moveto (pdf, x[i], y[i])
		ELSE
			cpdf_lineto (pdf, x[i], y[i])
		END IF
	NEXT i
	cpdf_stroke (pdf)

END FUNCTION
END PROGRAM
