'
'
' ####################
' #####  PROLOG  #####
' ####################
'
' Draws all marker symbols, filled and open.
' Also draws pointer arrows.
'
PROGRAM	"markers"
VERSION	"0.0001"
CONSOLE

'	IMPORT	"xst"   ' Standard library : required by most programs
'	IMPORT	"xio"		' Console IO library
	IMPORT	"cpdf"	' cpdf.dll

DECLARE FUNCTION  Entry ()
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION  Entry ()

	SINGLE XBUMP, x, y1, y2, y0, x0, markersize

	$NUMMARKERS = 11
	x  = 1.0
	y1 = 1.0
	y2 = 1.0
	y0 = 1.0
	x0 = 1.0

	XBUMP	= 0.35
	markersize = 12.0

' ***** Initialization *****

	pdf = cpdf_open (0, NULL)
	cpdf_enableCompression (pdf, $$YES)												' use Flate/Zlib compression
	cpdf_setCreator (pdf, &"ClibPDF Demo Program: markers.x")
	cpdf_init (pdf)
	cpdf_pageInit (pdf, 1, $$PORTRAIT, &$$LETTER, &$$LETTER)	' page orientation
	cpdf_translate (pdf, 0.0, 6.5)

' ***** Your drawing code in PDF below *****

' draw dashed lines to define reference points at which markers are positioned.
	cpdf_setdash (pdf, &"[2 2] 0")
	cpdf_setlinewidth (pdf, 0.12)
	y1 = 1.3
	cpdf_moveto (pdf, 1.1, y1)
	cpdf_lineto (pdf, 6.9, y1)
	y2 = 1.6
	cpdf_moveto (pdf, 1.1, y2)
	cpdf_lineto (pdf, 6.9, y2)

	y0 = 1.45
	FOR i = 0 TO $NUMMARKERS + 3
		IF (i < $NUMMARKERS) THEN
			x = XBUMP * i + XBUMP + x0
		ELSE
			x = XBUMP * i + XBUMP*2.0 + x0
		END IF
		cpdf_moveto (pdf, x, y0+0.45)
		cpdf_lineto(pdf, x, y0-0.45)
	NEXT i
	cpdf_stroke (pdf)
	cpdf_nodash (pdf)

' Draw markers and pointers at intersections defined by dashed lines
' First do filled (black) symbols
	cpdf_setgrayStroke (pdf, 0.0)
	cpdf_setgrayFill (pdf, 0.0)
	FOR i = 0 TO $NUMMARKERS + 3
		IF (i < $NUMMARKERS) THEN
			x = XBUMP * i + XBUMP + x0
			cpdf_marker (pdf, x, y1, i, markersize)	' filled
		ELSE
			x = XBUMP * i + XBUMP*2 + x0
			cpdf_pointer (pdf, x, y1, i-$NUMMARKERS, markersize)	' filled
		END IF
	NEXT i

' Do open symbols
	cpdf_setgrayStroke (pdf, 0.0)
	cpdf_setgrayFill (pdf, 1.0)
	FOR i = 0 TO $NUMMARKERS + 3
		IF (i < $NUMMARKERS) THEN
			x = XBUMP * i + XBUMP + x0
			cpdf_marker (pdf, x, y2, i, markersize)	' open
		ELSE
			x = XBUMP * i + XBUMP*2 + x0
			cpdf_pointer(pdf, x, y2, i-$NUMMARKERS, markersize)	' open
		END IF
	NEXT i

' Show symbol number above corresponding symbols
	cpdf_setgrayFill (pdf, 0.0)		' black
	cpdf_beginText (pdf, 0)
	cpdf_setFont (pdf, &"Helvetica", &"MacRomanEncoding", 12.0)
	FOR i = 0 TO $NUMMARKERS + 3
		IF (i < $NUMMARKERS) THEN
			x = XBUMP * i + XBUMP + x0
			IF (i == 10) THEN	x = x - 0.06
	    text$ = STRING$ (i)
		ELSE
			x = XBUMP * i + XBUMP*2 + x0
			text$ = STRING$ (i-$NUMMARKERS)
		END IF
		cpdf_text (pdf, x-0.05, y0+0.52, 0.0, &text$)
	NEXT i
	cpdf_endText (pdf)

	cpdf_beginText (pdf, 0)
	cpdf_setFont (pdf, &"Times-Roman", &"MacRomanEncoding", 14.0)
	cpdf_text (pdf, 1.2, 2.6, 0.0, &"markers.x  --   markersize = 12 points")
	cpdf_text (pdf, 2.55, 2.24, 0.0, &"cpdf_marker()")
	cpdf_text (pdf, 5.48, 2.24, 0.0, &"cpdf_pointer()")
	cpdf_endText (pdf)


	cpdf_finalizeAll (pdf)		' PDF file/memstream is actually written here
	cpdf_savePDFmemoryStreamToFile (pdf, &"markers.pdf")

' ***** Clean up *****

	cpdf_launchPreview(pdf)		' launch Acrobat/PDF viewer on the output file
	cpdf_close (pdf)					' shut down

	RETURN

END FUNCTION
END PROGRAM
