' 
' 
' ####################
' #####  PROLOG  #####
' ####################
' 
' PDF clock based on current time
' 
PROGRAM "pdfclock"
VERSION "0.0001"
CONSOLE

IMPORT "xst"		' Standard library : required by most programs
IMPORT "xsx"
IMPORT "xio"		' Console IO library
IMPORT "cpdf"		' cpdf.dll
IMPORT "msvcrt"		' msvcrt.dll

DECLARE FUNCTION Entry ()
DECLARE FUNCTION drawClockFace (pdf)
DECLARE FUNCTION drawHand (pdf, SINGLE angle, SINGLE length, SINGLE width, SINGLE r, SINGLE g, SINGLE b)

$$DEGPERHOUR = 30.0
$$DEGPERMIN = 6.0
$$DEGPERSEC = 6.0
$$XORIG = 4.25
$$YORIG = 7.0
$$FACERADIUS = 2.5
$$HOURHLENGTH = 1.8
$$MINHLENGTH = 2.3
$$SECHLENGTH = 2.4
$$HOURHWIDTH = 16.0
$$MINHWIDTH = 12.0
$$SECHWIDTH = 4.0
$$DEG2RADIAN = 0.0174532925
' 
' 
' ######################
' #####  Entry ()  #####
' ######################
' 
FUNCTION Entry ()

	TM tme
	SINGLE hourangle, minangle, secangle
	
	' == Initialization ==
	pdf = cpdf_open (0, NULL)
	cpdf_enableCompression (pdf, $$YES)		' use Flate/Zlib compression
	cpdf_init (pdf)
	cpdf_setTitle (pdf, &"PDF clock CGI")
	cpdf_setSubject (pdf, &"PDF clock CGI - generates PDF with analog and digital clock")
	cpdf_pageInit (pdf, 1, $$PORTRAIT, &$$LETTER, &$$LETTER)		' create page

	' == PDF drawing here ===
	drawClockFace (pdf)

	' Get current time and compute angles of clock hands
	t = time (0)
	tsAddr = ctime (&t)
	strbuf$ = CSTRING$ (tsAddr)
	memcpy (&tme, localtime (&t), SIZE (tme))
		
	hourangle = 90.0 - $$DEGPERHOUR * (SINGLE (tme.tm_hour MOD 12) + (SINGLE (tme.tm_min) / 60.0))
	minangle = 90.0 - $$DEGPERMIN * (SINGLE (tme.tm_min) + (SINGLE (tme.tm_sec) / 60.0))
	secangle = 90.0 - $$DEGPERSEC * tme.tm_sec

	IF (hourangle < -180.0) THEN hourangle = hourangle + 360.0
	IF (minangle < -180.0) THEN minangle = minangle + 360.0
	IF (secangle < -180.0) THEN secangle = secangle + 360.0

	' draw clock hands
	cpdf_setlinecap (pdf, 1)		' round cap for hands
	drawHand (pdf, hourangle, $$HOURHLENGTH, $$HOURHWIDTH, 0.5, 0.2, 0.0)		' hour
	drawHand (pdf, minangle, $$MINHLENGTH, $$MINHWIDTH, 0.5, 0.2, 0.0)		' minute
	drawHand (pdf, secangle, $$SECHLENGTH, $$SECHWIDTH, 0.3, 0.3, 1.0)		' second
	cpdf_setlinecap (pdf, 0)

	' Show Time/Date in text
	cpdf_setrgbcolor (pdf, 1.0, 0.0, 0.0)
	cpdf_beginText (pdf, 0)
	cpdf_setFont (pdf, &"Times-Roman", &"MacRomanEncoding", 34.0)
	cpdf_textAligned (pdf, $$XORIG, 9.85, 0.0, $$TEXTPOS_LM, &strbuf$)
	cpdf_endText (pdf)

	' some notes
	cpdf_setgray (pdf, 0.0)
	cpdf_beginText (pdf, 0)
	cpdf_setFont (pdf, &"Times-Italic", &"MacRomanEncoding", 24.0)
	cpdf_textAligned (pdf, $$XORIG, 4.0, 0.0, $$TEXTPOS_LM, &"[California Time]")
	cpdf_textAligned (pdf, $$XORIG, 3.55, 0.0, $$TEXTPOS_LM, &"Dynamic PDF CGI Demo")

	IF (getenv (&"REMOTE_HOST")) THEN
		strncpy (strbuf, getenv (&"REMOTE_HOST"), 60)
		cpdf_textAligned (pdf, $$XORIG, 3.1, 0.0, $$TEXTPOS_LM, &strbuf$)
	ELSE
		cpdf_textAligned (pdf, $$XORIG, 3.1, 0.0, $$TEXTPOS_LM, &"Remote hostname when run as CGI")
	END IF

	IF (getenv (&"REMOTE_ADDR")) THEN
		strncpy (&strbuf$, getenv (&"REMOTE_ADDR"), 60)
		cpdf_textAligned (pdf, $$XORIG, 2.65, 0.0, $$TEXTPOS_LM, &strbuf$)
	ELSE
		cpdf_textAligned (pdf, $$XORIG, 2.65, 0.0, $$TEXTPOS_LM, &"Remote host IP when run as CGI")
	END IF

	cpdf_endText (pdf)

	' == generate output and clean up ==
	cpdf_finalizeAll (pdf)		' PDF file/memstream is actually written here

	' next two lines added to see output if created as .exe
	cpdf_savePDFmemoryStreamToFile (pdf, &"pdfclock.pdf")
	cpdf_launchPreview (pdf)		' launch Acrobat/PDF viewer on the output file

	' Write to stdout with proper content specification
	bufPDF = cpdf_getBufferForPDF (pdf, &length)

	_setmode (1, $$_O_BINARY)		' for binary stdout in Windows

	IF (getenv (&"REMOTE_ADDR")) THEN		' do this only when running as a CGI
		printf (&"Content-Type: application/pdf%c", 10)
		printf (&"Content-Length: %d%c%c", length, 10, 10)
	END IF
	fwrite (bufPDF, 1, length, 1)		' Send PDF now
	fflush (1)

	' == Clean up ==
	cpdf_close (pdf)		' shut down
	RETURN
END FUNCTION

FUNCTION drawHand (pdf, SINGLE angle, SINGLE length, SINGLE width, SINGLE r, SINGLE g, SINGLE b)

	SINGLE xh, yh
	
	cpdf_setrgbcolor (pdf, r, g, b)
	cpdf_setlinewidth (pdf, width)
	xh = length * cos ($$DEG2RADIAN * angle)
	yh = length * sin ($$DEG2RADIAN * angle)
	cpdf_moveto (pdf, $$XORIG, $$YORIG)
	cpdf_lineto (pdf, xh + $$XORIG, yh + $$YORIG)
	cpdf_stroke (pdf)
END FUNCTION

FUNCTION drawClockFace (pdf)

	SINGLE xb, yb, xe, ye

	cpdf_setlinewidth (pdf, 5.0)
	cpdf_setgray (pdf, 0.0)
	cpdf_setrgbcolorFill (pdf, 0.85, 0.93, 0.7)
	cpdf_circle (pdf, $$XORIG, $$YORIG, $$FACERADIUS)
	cpdf_fillAndStroke (pdf)

	' hour ticks */
	FOR i = 0 TO 11		'for(i=0 i<12 i++) {
		xe = $$FACERADIUS * cos ($$DEGPERHOUR * $$DEG2RADIAN * SINGLE (i))
		ye = $$FACERADIUS * sin ($$DEGPERHOUR * $$DEG2RADIAN * SINGLE (i))
		xb = xe * $$MINHLENGTH / $$FACERADIUS
		yb = ye * $$MINHLENGTH / $$FACERADIUS
		cpdf_moveto (pdf, xb + $$XORIG, yb + $$YORIG)
		cpdf_lineto (pdf, xe + $$XORIG, ye + $$YORIG)
	NEXT i
	cpdf_stroke (pdf)

	' min/second ticks */
	cpdf_setlinewidth (pdf, 2.0)
	FOR i = 0 TO 59		'for(i=0 i<60 i++) {
		xe = $$FACERADIUS * cos ($$DEGPERSEC * $$DEG2RADIAN * SINGLE (i))
		ye = $$FACERADIUS * sin ($$DEGPERSEC * $$DEG2RADIAN * SINGLE (i))
		xb = xe * $$SECHLENGTH / $$FACERADIUS
		yb = ye * $$SECHLENGTH / $$FACERADIUS
		cpdf_moveto (pdf, xb + $$XORIG, yb + $$YORIG)
		cpdf_lineto (pdf, xe + $$XORIG, ye + $$YORIG)
	NEXT i
	cpdf_stroke (pdf)
END FUNCTION


END PROGRAM