' 
' 
' ####################
' #####  PROLOG  #####
' ####################
' 
' 
' 
PROGRAM "timeaxis"
VERSION "0.0001"
CONSOLE

IMPORT "xst"		' Standard library : required by most programs
'IMPORT "xsx"
IMPORT "xio"		' Console IO library
IMPORT "cpdf"		' cpdf.dll
IMPORT "msvcrt"		' msvcrt.dll

DECLARE FUNCTION Entry ()

' 
' ######################
' #####  Entry ()  #####
' ######################
' 
FUNCTION Entry ()

	TM tm1, tm2
	SINGLE respmax
	SINGLE ybump, yoffset
	yearformat = $$YEAR_2DIGIT
	respmax = 10.0

	' == Initialization ==
	pdf = cpdf_open (0, NULL)
	cpdf_enableCompression (pdf, $$YES)		' use Flate/Zlib compression
	cpdf_init (pdf)
	cpdf_pageInit (pdf, 1, $$PORTRAIT, &$$LETTER, &$$LETTER)		' page orientation
	cpdf_translate (pdf, 0.0, 1.0)
	' ----------------------------------------------------------------------------------

	' Time domain version
	t = time (0)
	' memcpy(&tm, localtime(&t), sizeof(struct tm))
	tm1.tm_sec   = 0
	tm1.tm_min   = 0
	tm1.tm_hour  = 0
	tm1.tm_mday  = 1		' day starts at 1
	tm1.tm_mon   = 0		' month starts at 0
	tm1.tm_year  = 125	' year since 1900
	tm1.tm_isdst = -1		' This is needed.
	mktime (&tm1)		    ' this rebuilds all the fields
	ybump = 0.5 * $$inch

	' -- start the first axis with 4 hour span
	memcpy (&tm2, &tm1, SIZE (tm1))
	tm2.tm_hour = tm2.tm_hour + 4		' 4 hour
	mktime (&tm2)		                ' this rebuilds all the fields
	yoffset = - 0.2 * $$inch

	' Creat time plot domain and use previously created time axis
	timeDomain = cpdf_createTimePlotDomain (1.0 * $$inch, 8.0 * $$inch, 4.0 * $$inch, 0.5 * $$inch, &tm1, &tm2, 0.0, respmax, $$TIME, $$LINEAR, 0)
	oldDomain = cpdf_setPlotDomain (pdf, timeDomain)		' save oldDomain for later restore
	' cpdf_fillDomainWithGray(timeDomain, 0.85)				' white
	cpdf_drawMeshForDomain (timeDomain)

	cpdf_setgray (pdf, 0.0)
	tAxis = cpdf_createTimeAxis (0.0, 4.0 * $$inch, $$TIME, &tm1, &tm2)
	cpdf_attachAxisToDomain (tAxis, timeDomain, 0.0, yoffset)
	cpdf_setTimeAxisNumberFormat (tAxis, $$MONTH_NAME, $$YEAR_2DIGIT, &"Helvetica", 11.0)
	' YEAR_FULL for 2025 or YEAR_2DIGIT for 25
	' cpdf_setAxisLabel(tAxis, "Date", "Helvetica-BoldOblique", "MacRomanEncoding", 14.0)
	cpdf_drawAxis (tAxis)
	cpdf_freeAxis (tAxis)
	' -- First axis done

	' -- Other axes --
	FOR i = 0 TO 3
		memcpy (&tm2, &tm1, SIZE (tm1))		' freshen tm2
		SELECT CASE i

			CASE 0 :
				tm2.tm_hour = tm2.tm_hour + 24		' 24 hours
						
			CASE 1 :
				tm2.tm_mday = tm2.tm_mday + 4		' 4 days

			CASE 2 :
				tm2.tm_mon = tm2.tm_mon + 6		' 6 months

			CASE 3 :
				tm2.tm_year = tm2.tm_year + 4		' 4 years
				yearformat = $$YEAR_FULL		' use 4 digits
			
		END SELECT

		tm2.tm_isdst = -1		' This is needed.
		mktime (&tm2)		' this rebuilds all the fields
		yoffset = yoffset - ybump

		' We reuse the domain, as we don't actually plot anything other than axis
		cpdf_setgray (pdf, 0.0)
		tAxis = cpdf_createTimeAxis (0.0, 4.0 * $$inch, $$TIME, &tm1, &tm2)
		cpdf_attachAxisToDomain (tAxis, timeDomain, 0.0, yoffset)
		cpdf_setTimeAxisNumberFormat (tAxis, $$MONTH_NAME, yearformat, &"Helvetica", 11.0)
		' YEAR_FULL for 1998 or YEAR_2DIGIT for 98
		' cpdf_setAxisLabel(tAxis, "Date", "Helvetica-BoldOblique", "MacRomanEncoding", 14.0)
		cpdf_drawAxis (tAxis)
		cpdf_freeAxis (tAxis)
		' -- one axis done
	NEXT i
	
	cpdf_setPlotDomain (pdf, oldDomain)		' restore old domain
	cpdf_freePlotDomain (timeDomain)		' deallocate the plot domain

	' ----------------------------------------------------------------------------------
	cpdf_finalizeAll (pdf)		' PDF file/memstream is actually written here
	cpdf_savePDFmemoryStreamToFile (pdf, &"timeaxis.pdf")

	' == Clean up ==
	cpdf_launchPreview (pdf)		' launch Acrobat/PDF viewer on the output file
	cpdf_close (pdf)		' shut down
		
	' a$ = INLINE$ ("press now")
	RETURN

END FUNCTION


END PROGRAM