' 
' 
' ####################
' #####  PROLOG  #####
' ####################
' 
' 
' 
PROGRAM "weather"
VERSION "0.0001"
CONSOLE

IMPORT "xst"		' Standard library : required by most programs
IMPORT "xsx"
IMPORT "xio"		' Console IO library
IMPORT "cpdf"		' cpdf.dll
IMPORT "msvcrt"		' msvcrt.dll

DECLARE FUNCTION Entry ()
DECLARE FUNCTION plot_temp (pdf, useC)
DECLARE FUNCTION plot_rHumidity (pdf)
DECLARE FUNCTION do_TextStuff (pdf, useC)
DECLARE FUNCTION read_datafile (gafile, useC)

$$CITYNAME = "OAKLAND"
$$MAXDATA = 1500

' 
' ######################
' #####  Entry ()  #####
' ######################
' 
FUNCTION Entry ()

	SHARED Ndata		' # data points
	SHARED temp[$$MAXDATA]		' temp F
	SHARED humidity[$$MAXDATA]
	SHARED TM xtime[$$MAXDATA]
	SHARED TM axStart, axEnd, recStart, recEnd
	SHARED tAxis
	SHARED sky$
	SHARED tempUnit$[2]

	tempUnit$[0] = "degrees F"
	tempUnit$[1] = "degrees C"
	tempUnit$[2] = ""

	' Some plot data and globals for settings

	useCentigrade = 0		' if non-zero, plot in centigrade

	file$ = "OAK.wdat"
	read_datafile (&file$, useCentigrade)

	' == OK, data file read in. Start plotting
	pdf = cpdf_open (0, NULL)
	cpdf_enableCompression (pdf, 1)		' use Flat/Zlib compression
	cpdf_init (pdf)
	cpdf_pageInit (pdf, 1, $$PORTRAIT, &$$LETTER, &$$LETTER)		' page orientation

	' PDF Info object
	cpdf_setCreator (pdf, &"weather.x")
	cpdf_setTitle (pdf, &"Weather of OAK California")
	cpdf_setSubject (pdf, &"Weather of Oakland International Airport, California")
	cpdf_setKeywords (pdf, &"Weather")

	' Your drawing code in PDF below
	' -----------------------------------------------------------------------------------------

	' plot curves
	plot_temp (pdf, useCentigrade)
	plot_rHumidity (pdf)
	do_TextStuff (pdf, useCentigrade)

	' -----------------------------------------------------------------------------------------
	' Your drawing code in PDF above
	cpdf_finalizeAll (pdf)		' PDF file is actually written here
	cpdf_savePDFmemoryStreamToFile (pdf, &"weather.pdf")

	cpdf_launchPreview (pdf)	' launch Acrobat/PDF viewer on the output file
	cpdf_close (pdf)					' PDF file is actually written here
	
END FUNCTION


FUNCTION plot_temp (pdf, useC)

	SHARED Ndata
	SHARED temp[]
	SHARED humidity[]
	SHARED TM xtime[]
	SHARED TM axStart, axEnd, recStart, recEnd
	SHARED tAxis

	SINGLE xt
	SINGLE vtmax, vtmin, recmin, recmax

	vtmax = -999
	vtmin = 9999
	recmin = 50
	recmax = 100

	' get data max and min values
	FOR i = 0 TO Ndata - 1		'(i=0 i<Ndata i++) {
		IF (temp[i] > vtmax) THEN vtmax = temp[i]
		IF (temp[i] < vtmin) THEN vtmin = temp[i]
	NEXT i

	cpdf_suggestMinMaxForLinearDomain (vtmin, vtmax, &recmin, &recmax)

	memcpy (&axStart, &xtime[0], SIZE (TM))
	axStart.tm_sec = 0
	axStart.tm_min = 0
	axStart.tm_hour = 0
	mktime (&axStart)		' this rebuilds all the fields
	memcpy (&axEnd, &xtime[Ndata - 1], SIZE (TM))
	axEnd.tm_sec = 0
	axEnd.tm_min = 0
	axEnd.tm_hour = 0
	' axEnd.tm_mday++
	INC axEnd.tm_mday
	mktime (&axEnd)		' this rebuilds all the fields
	
	cpdf_suggestTimeDomainParams (&axStart, &axEnd, &recStart, &recEnd)
	tempDomain = cpdf_createTimePlotDomain (1.7 * $$inch, 5.5 * $$inch, 6.0 * $$inch, 2.5 * $$inch, &recStart, &recEnd, recmin, recmax, $$TIME, $$LINEAR, 0)
	oldDomain = cpdf_setPlotDomain (pdf, tempDomain)		' save oldDomain for later restore
	IF (useC) THEN
		cpdf_setLinearMeshParams (tempDomain, $$Y_MESH, recmin, 5.0, recmin, 1.0)
	ELSE
		cpdf_setLinearMeshParams (tempDomain, $$Y_MESH, recmin, 10.0, recmin, 5.0)
	END IF
		
	cpdf_fillDomainWithRGBcolor (tempDomain, 1.0, 0.9, 0.9)		' color of domain: light pink
	cpdf_drawMeshForDomain (tempDomain)

	cpdf_setgray (pdf, 0.0)

	' X-Axis ---------------------------------------------------------------------------------
	tAxis = cpdf_createTimeAxis (0.0, 6.0 * $$inch, $$TIME, &recStart, &recEnd)
	cpdf_attachAxisToDomain (tAxis, tempDomain, 0.0, - 0.2 * $$inch)
	cpdf_setTimeAxisNumberFormat (tAxis, $$MONTH_NAME, $$YEAR_2DIGIT, &"Helvetica", 12.0)
	' YEAR_FULL for 1998 or YEAR_2DIGIT for 98
	cpdf_setAxisLabel (tAxis, &"Date", &"Times-Roman", &"MacRomanEncoding", 16.0)
	cpdf_drawAxis (tAxis)
	' cpdf_freeAxis(tAxis) 	' we use it again for plotting other things

	' Y-Axis ---------------------------------------------------------------------------------
	yAxis = cpdf_createAxis (90.0, 2.5 * $$inch, $$LINEAR, recmin, recmax)
	cpdf_attachAxisToDomain (yAxis, tempDomain, - 0.2 * $$inch, 0.0)		' give a little X offset left
	cpdf_setAxisNumberFormat (yAxis, &"%.f", &"Helvetica", 12.0)
	IF (useC) THEN
		cpdf_setAxisLabel (yAxis, &"Temperature (degrees C)", &"Times-Roman", &"MacRomanEncoding", 16.0)
	ELSE
		cpdf_setAxisLabel (yAxis, &"Temperature (degrees F)", &"Times-Roman", &"MacRomanEncoding", 16.0)
	END IF
	

	IF (useC) THEN
		cpdf_setLinearAxisParams (yAxis, recmin, 5.0, recmin, 1.0)
	ELSE
		cpdf_setLinearAxisParams (yAxis, recmin, 10.0, recmin, 5.0)
	END IF
	
	cpdf_drawAxis (yAxis)
	cpdf_freeAxis (yAxis)

	' Do the tuning curve first
	cpdf_comments (pdf, &"\n%% plot the temperature curve.\n")
	cpdf_setlinewidth (pdf, 1.5)
	cpdf_setrgbcolorStroke (pdf, 1.0, 0.0, 0.0)		' red

	FOR i = 0 TO Ndata - 1
		xt = tm_to_NumDays (&recStart, &xtime[i])
		IF (i == 0) THEN
			cpdf_moveto (pdf, xt, temp[i])
		ELSE
			cpdf_lineto (pdf, xt, temp[i])
		END IF
	NEXT i

	cpdf_setlinejoin (pdf, 1)
	cpdf_stroke (pdf)
	cpdf_setlinejoin (pdf, 0)

	cpdf_setPlotDomain (pdf, oldDomain)		' restore previous plot domain
	cpdf_freePlotDomain (tempDomain)		' deallocate the plot domain
END FUNCTION


FUNCTION plot_rHumidity (pdf)

	SHARED Ndata
	SHARED temp[]
	SHARED humidity[]
	SHARED TM xtime[]
	SHARED TM axStart, axEnd, recStart, recEnd
	SHARED tAxis

	SINGLE xt
	SINGLE vtmax, vtmin, recmin, recmax

	vtmax = -999.0
	vtmin = 9999.0
	recmin = 0.0
	recmax = 100.0

	' get data max and min values
	FOR i = 0 TO Ndata - 1
		IF (humidity[i] > vtmax) THEN vtmax = humidity[i]
		IF (humidity[i] < vtmin) THEN vtmin = humidity[i]
	NEXT i

	cpdf_suggestMinMaxForLinearDomain (vtmin, vtmax, &recmin, &recmax)
	cpdf_suggestTimeDomainParams (&axStart, &axEnd, &recStart, &recEnd)
	rhDomain = cpdf_createTimePlotDomain (1.7 * $$inch, 1.7 * $$inch, 6.0 * $$inch, 2.5 * $$inch, &recStart, &recEnd, recmin, recmax, $$TIME, $$LINEAR, 0)
	oldDomain = cpdf_setPlotDomain (pdf, rhDomain)		' save oldDomain for later restore

	cpdf_setLinearMeshParams (rhDomain, $$Y_MESH, recmin, 10.0, recmin, 5.0)
	cpdf_fillDomainWithRGBcolor (rhDomain, 1.0, 0.9, 0.9)		' color of domain: light pink
	cpdf_drawMeshForDomain (rhDomain)

	cpdf_setgray (pdf, 0.0)

	' X-Axis - we already have the time axis for temp plot -----------------------------------
	' tAxis = cpdf_createTimeAxis(0.0, 6.0*inch, TIME, &recStart, &recEnd)
	cpdf_attachAxisToDomain (tAxis, rhDomain, 0.0, - 0.2 * $$inch)
	' cpdf_setTimeAxisNumberFormat(tAxis, MONTH_NAME, YEAR_2DIGIT, "Helvetica", 12.0)
	' YEAR_FULL for 1998 or YEAR_2DIGIT for 98
	' cpdf_setAxisLabel(tAxis, "Date", "Times-Roman", "MacRomanEncoding", 16.0)
	cpdf_drawAxis (tAxis)
	cpdf_freeAxis (tAxis)		' we are done, no more use

	' Y-Axis ---------------------------------------------------------------------------------
	yAxis = cpdf_createAxis (90.0, 2.5 * $$inch, $$LINEAR, recmin, recmax)
	cpdf_attachAxisToDomain (yAxis, rhDomain, - 0.2 * $$inch, 0.0)		' give a little X offset left
	cpdf_setAxisNumberFormat (yAxis, &"%.f", &"Helvetica", 12.0)
	cpdf_setAxisLabel (yAxis, &"Humidity (%)", &"Times-Roman", &"MacRomanEncoding", 16.0)

	cpdf_setLinearAxisParams (yAxis, recmin, 10.0, recmin, 5.0)
	cpdf_drawAxis (yAxis)
	cpdf_freeAxis (yAxis)

	' Do the tuning curve first
	cpdf_comments (pdf, &"\n%% plot the humidity curve.\n")
	cpdf_setlinewidth (pdf, 1.5)
	cpdf_setrgbcolorStroke (pdf, 0.0, 0.0, 1.0)		' blue

	FOR i = 0 TO Ndata - 1		'for(i=0 i<Ndata i++) {
		xt = tm_to_NumDays (&recStart, &xtime[i])
		IF (i == 0) THEN
			cpdf_moveto (pdf, xt, humidity[i])
		ELSE
			cpdf_lineto (pdf, xt, humidity[i])
		END IF
	NEXT i
	cpdf_setlinejoin (pdf, 1)
	cpdf_stroke (pdf)
	cpdf_setlinejoin (pdf, 0)

	cpdf_setPlotDomain (pdf, oldDomain)		' restore previous plot domain
	cpdf_freePlotDomain (rhDomain)		' deallocate the plot domain
END FUNCTION


' --------------------------------------------------------------------------------
' Key words for data file entries

FUNCTION read_datafile (gafile, useC)

	SHARED Ndata	
	SHARED TM xtime[]
	SHARED temp[]
	SHARED humidity[]
	SHARED sky$

	' SINGLE vt, vdew, vrh

	linebuf$ = NULL$ (1024)
	strbuf1$ = NULL$ (128)
	sky$ = NULL$ (16)

	fp = fopen (gafile, &"r")

	IF (fp == NULL) THEN
		fprintf (stdout, &"Can't open data file: %s\n", gafile)
		QUIT (1)
	END IF

	Ndata = 0
	DO WHILE ((fgets (&linebuf$, 1020, fp)) <> 0) && (Ndata <= $$MAXDATA - 1)
		IF (linebuf${0} == '#' || linebuf${0} == '%' || LEN (CSIZE$ (linebuf$)) < 3) THEN DO DO
		' continue		' pretty much blank line
		sscanf (&linebuf$, &"%s", &strbuf1$)		' first word
		IF (strstr (&strbuf1$, &"19") || strstr (&strbuf1$, &"20")) THEN
			' date field
			sscanf (&linebuf$, &"%d-%d-%d %d:%d:%d", &year, &month, &day, &hour, &minute, &second)
			year = year - 1900
			xtime[Ndata].tm_year = year
			xtime[Ndata].tm_mon = month - 1
			xtime[Ndata].tm_mday = day
			xtime[Ndata].tm_hour = hour
			xtime[Ndata].tm_min = minute
			xtime[Ndata].tm_sec = second
			' mktime(&xtime[Ndata])		' this rebuilds all the fields

		ELSE
			IF (strncmp (&strbuf1$, &$$CITYNAME, 3) == 0) THEN
				vt = -999 : vdew = -999 : vrh = -999		' init to invalid values
				sscanf (&linebuf$ + 15, &"%s %d %d %d", &sky$, &vt, &vdew, &vrh)

				IF (strncmp (&sky$, &"NOT", 3) == 0 || strncmp (&sky$, &"N/A", 3) == 0 || strncmp (&sky$, &"MISG", 4) == 0 || strncmp (&sky$, &"MM", 2) == 0) THEN DO DO
				IF (vt < -990 || vdew < -990 || vrh < -990) THEN DO DO

				' OK, we are pretty sure we have valid data
				IF (useC) THEN
					temp[Ndata] = 0.5555555 * (SINGLE (vt) - 32.0)		' convert to centigrade
				ELSE
					temp[Ndata] = vt
				END IF
				humidity[Ndata] = vrh
				INC Ndata
			END IF
		END IF
	LOOP

	fclose (fp)
	RETURN
END FUNCTION


FUNCTION do_TextStuff (pdf, useC)

	SHARED Ndata
	SHARED TM xtime[]
	SHARED temp[]
	SHARED humidity[]
	SHARED tempUnit$[]
	SHARED sky$

	i = Ndata - 1

	tbuf$ = NULL$ (1024)
	cpdf_setgray (pdf, 0.0)
	cpdf_beginText (pdf, 0)
	cpdf_setFont (pdf, &"Helvetica-Bold", &"MacRomanEncoding", 18.0)
	cpdf_text (pdf, 1.5, 9.8, 0.0, &"Weather of Oakland International Airport, California")
	cpdf_endText (pdf)

	cpdf_setgray (pdf, 0.0)
	cpdf_beginText (pdf, 0)
	cpdf_setFont (pdf, &"Helvetica", &"MacRomanEncoding", 14.0)
	sprintf (&tbuf$, &"Date: %d-%d-%d     Time: %2d:%2d", xtime[i].tm_year + 1900, xtime[i].tm_mon + 1, xtime[i].tm_mday, xtime[i].tm_hour, xtime[i].tm_min)
	cpdf_text (pdf, 1.7, 9.4, 0.0, &tbuf$)

	tbuf$ = NULL$ (1024)
	sprintf (&tbuf$, &"Temperature: %3d %s,    Relative Humidity: %2d%%", temp[i], &tempUnit$[useC], humidity[i])
	cpdf_text (pdf, 1.7, 9.1, 0.0, &tbuf$)
	sky$ = CSIZE$ (sky$)
	sky$ = "Sky: " + sky$
	' sprintf(tbuf, &"Sky: %s", &sky$)
	cpdf_text (pdf, 1.7, 8.8, 0.0, &sky$)
	cpdf_endText (pdf)

END FUNCTION

END PROGRAM